@@ -392,8 +392,8 @@ sub find_scsi_controller_device {
    my $vm = $args{vm};
    my $devices = $vm->config->hardware->device;
    foreach my $device (@$devices) {
-      my $class = ref $device;
-      if($class->isa('VirtualLsiLogicController')) {
+      my $hardware = $device->deviceInfo->{label};
+      if($hardware =~ /SCSI controller/i) {
          return $device;
       }
    }
