@@ -228,7 +228,8 @@ sub do_browse() {
                   print  $_."\n";
                }
             }
-            if ($#{@vm_files} == 0) {
+            my $file_check = @vm_files;
+            if ($file_check == 0) {
                print "No files found under this location\n";
             }
             $temp = $remotepath;
