@@ -1,9 +1,9 @@
 #!/usr/bin/perl -w
 #
-# Copyright (c) 2007 VMware, Inc.  All rights reserved.
+# Copyright (c) 2007-2017 VMware, Inc.  All rights reserved.
 #
 
-## bug 299795 start all guestOS changed to guestos and IPaddress changed to ipaddress 
+## bug 299795 start all guestOS changed to guestos and IPaddress changed to ipaddress
 use strict;
 use warnings;
 
@@ -138,17 +138,17 @@ Util::connect();
 
 if ($operation eq 'list') {
   list_snapshot();
-} if ($operation eq 'create') {
+} elsif ($operation eq 'create') {
   create_snapshot();
-} if ($operation eq 'revert') {
+} elsif ($operation eq 'revert') {
   revert_snapshot();
-} if ($operation eq 'goto') {
+} elsif ($operation eq 'goto') {
   goto_snapshot();
-} if ($operation eq 'rename') {
+} elsif ($operation eq 'rename') {
   rename_snapshot();
-} if ($operation eq 'removeall') {
+} elsif ($operation eq 'removeall') {
   remove_all_snapshot();
-} if ($operation eq 'remove') {
+} elsif ($operation eq 'remove') {
   remove_snapshot();
 }
 
