@@ -204,6 +204,9 @@ sub create_vm {
           elsif (ref($@->detail) eq 'DuplicateName') {
              Util::trace(0, "Virtual machine already exists.\n");
           }
+          elsif (ref($@->detail) eq 'CannotAccessFile'){
+             Util::trace(0, "Unable to access datastore file.\n");
+          }
           else {
              Util::trace(0, "\n" . $@ . "\n");
           }
@@ -219,7 +222,7 @@ sub create_vm {
 # ================================================
 sub create_conf_spec {
    my $controller =
-      VirtualBusLogicController->new(key => 0,
+      VirtualLsiLogicController->new(key => 0,
                                      device => [0],
                                      busNumber => 0,
                                      sharedBus => VirtualSCSISharing->new('noSharing'));
