@@ -1,6 +1,6 @@
 
 #
-# Copyright 2006 VMware, Inc.  All rights reserved.
+# Copyright 2006,2016,2017 VMware, Inc.  All rights reserved.
 #
 
 use 5.006001;
@@ -9,6 +9,12 @@ use warnings;
 
 use Carp qw(confess croak);
 use XML::LibXML;
+use Config;
+use MIME::Base64;
+if (-f "/etc/redhat-release"){
+   use lib "/usr/lib64/perl5";
+   use lib "/usr/lib64/perl5/libwww-perl-5.837/lib";
+}
 use LWP::UserAgent;
 use LWP::ConnCache;
 use HTTP::Request;
@@ -16,7 +22,7 @@ use HTTP::Headers;
 use HTTP::Response;
 use HTTP::Cookies;
 use Data::Dumper;
-
+use VMware::SSOConnection;
 
 ##################################################################################
 package Util;
@@ -51,27 +57,90 @@ sub fail {
 sub connect {
    # url should be <protocol>://<hostname>/sdk
    my ($url, $username, $password) = @_;
-   
    # bug 217340
    @_ = ();
    my $retval = undef;
-   
+   my $ssourl = undef;
+   my $lsurl = undef;
+   my $vcUrl = undef;
+   my $pscurl = Opts::get_option('psc');
+   if (defined($pscurl)){
+      $lsurl = "https://". $pscurl . "/lookupservice/sdk";
+      my $lookupservice = new LookupService(lookup_url => $lsurl);
+      $ssourl = $lookupservice->find_sso_url($lsurl);
+      my %results = $lookupservice->get_default_mgmt_node();
+      my @nodes_ids = values %results;
+      my $node_id = $nodes_ids[0];
+      $url = $lookupservice->find_vim_url($node_id);
+   }
+
    if (!defined($url)) {
       $url = Opts::get_option('url');
    }
-   
    if (Opts::option_is_set('sessionfile')) {
       my $sessionfile = Opts::get_option('sessionfile');
       $retval = Vim::load_session(service_url => $url, session_file => $sessionfile);
+   } elsif (defined($ssourl)) {
+      if (!defined($username)) {
+         $username = Opts::get_option('username');
+      }
+
+      if (!defined($password)) {
+         $password = Opts::get_option('password');
+      }
+
+      # Constructing SSO url for IPv6 host
+      # e.g. If SSO host provided is in IPv6 format say, https://fe:10:20:30:70e:fe/sts/STSService
+      my $sso_host = $ssourl;
+      # Triming out, host name/IP from "sso_host" i.e. $2 = "fe:10:20:30:70e:fe"
+      if ($sso_host =~ s|http(s?)://(.*)/sts/.*|http$1://$2/sts(.*)|i) {
+         $sso_host = $2;
+         # Checking if the host specified is in IPv6 format
+         if (($sso_host =~ tr/:/:/) > 1){
+            if (!($sso_host =~ /]/)){
+               # If host is in IPv6 and doesen't have any square bracket then adding it
+               $sso_host = "[" . $sso_host . "]";
+               # Constructing sso url as "http(s)//[fe:10:20:30:70e:fe]/sts/STSService"
+               $ssourl =~ s|http(s?)://(.*)/sts.*|http$1://$sso_host/sts/STSService|i;
+            }
+         }
+      }
+      my $ssoConnection = VMware::SSOConnection->new(sso_url => $ssourl);
+      my $session_id = undef;
+      my $bearerToken = $ssoConnection->get_bearer_saml_token(user_name => $username, password => $password);
+      if (defined ($url)) {
+         my $vcUrl = $url;
+         if ($vcUrl =~ s|http(s?)://(.*)/sdk.*|http$1://$2/sdk(.*)|i) {
+            $vcUrl = $2;
+         }
+         if (($vcUrl =~ tr/:/:/) > 1){
+            $vcUrl =~ s/\[(.*)\](.*)/$1/;
+         }else {
+            # To remove port from hostname or IPv4 address
+            $vcUrl =~ s/:(.*)// ;
+         }
+         $session_id = $ssoConnection->connect(server => $vcUrl);
+      } else {
+         $session_id = $bearerToken;
+      }
+      if (!$session_id) {
+          die "Failed to get session cookie";
+      }
+      if (!defined($url)) {
+         Opts::set_option('url', $vcUrl);
+         $url = $vcUrl;
+      }
+
+      $retval = Vim::load_session(service_url => $url, session_id => $session_id);
    } elsif ((Opts::option_is_set('username') || defined($username)) && (Opts::option_is_set('password') || defined($password))) {
       if (!defined($username)) {
          $username = Opts::get_option('username');
       }
-      
+
       if (!defined($password)) {
          $password = Opts::get_option('password');
       }
-      
+
       eval {
          $retval = Vim::login(service_url => $url, user_name => $username, password => $password);
       };
@@ -360,10 +429,82 @@ sub new {
    my $url = delete($args{service_url});
 
    if (keys %args) {
-      croak "Unrecognized arg(s) " . 
+      croak "Unrecognized arg(s) " .
             join(', ', sort keys %args) . " to 'Vim::new'";
    }
 
+   #
+   # Block to remove Perl Bug related to Net-HTTP module: IPv6 Address URLs Broken.
+   # For details URL: https://rt.cpan.org/Public/Bug/Display.html?id=75618
+   #
+   my $portnumber = Opts::get_option('portnumber');
+   if (!$portnumber) {
+      $portnumber="443";
+   }
+
+   my $portflag = 0;
+   $url =~ s|http(s?)://(.*)/sdk.*|http$1://$2/sdk/vimService.wsdl|i;
+   my $server_from_url = $2;
+   # setting port, default 443 else from option "portnumber" or from url itself
+    if (defined Opts::get_option('portnumber')) {
+       $portflag = 1;
+       #for IPv6
+       if (($server_from_url =~ tr/:/:/) > 1){
+          my $temp = $server_from_url;
+          $temp =~ s,:(\d+)$,,;
+          if ($1 eq $portnumber){
+             $server_from_url =~ s,:(\d+)$,,;
+          }
+       }
+       else{
+          #for IPv4
+          $server_from_url =~ s,:(\d+)$,,;
+       }
+    }
+   # obtaining server from URL
+   if (($server_from_url =~ tr/:/:/) > 1) {
+      if ($server_from_url =~ /]]]/){
+         # do nothing
+      }
+      elsif($server_from_url =~ /]]/){
+         if (($server_from_url =~ s,:(\d+)$,,)) {
+            my $port_no = int($1);
+            $portflag = 1;
+            $portnumber = $port_no;
+            $server_from_url =~ s|(\[*]):(.*)|$1:$2|i;
+         }
+         if ($^O =~ /MSWin32/) {
+            $server_from_url = "[" . $server_from_url ."]";
+         }
+      }
+      elsif ($server_from_url =~ /]/){
+         if (($server_from_url =~ s,:(\d+)$,,)) {
+            my $port_no = int($1);
+            $portflag = 1;
+            $portnumber = $port_no;
+            $server_from_url =~ s|(\[*]):(.*)|$1:$2|i;
+         }
+         if ($^O =~ /MSWin32/) {
+            $server_from_url = "[[" . $server_from_url ."]]";
+         }
+      }
+      else {
+         if ($^O =~ /MSWin32/) {
+            $server_from_url = "[[[" . $server_from_url ."]]]";
+         }
+         else {
+            $server_from_url = "[" . $server_from_url ."]";
+         }
+      }
+   }
+   # constructing URL
+   if ($portflag == "1") {
+      $url =~ s|http(s?)://(.*)/sdk.*|http$1://$server_from_url:$portnumber/sdk/vimService.wsdl|i;
+   }
+   else {
+      $url =~ s|http(s?)://(.*)/sdk.*|http$1://$server_from_url/sdk/vimService.wsdl|i;
+   }
+
    $url = Opts::construct_service_url(
       server => $server,
       protocol => $protocol,
@@ -423,21 +564,21 @@ sub get_vim {
 sub query_api_supported {
    my $url = shift;
    my %supportedapiversions;
-   if ($url =~ s|http(s?)://(.*)/sdk.*|http$1://$2/sdk/vimService.wsdl|i) {      
-      if ($1 eq "s") {
-         eval {  
-            require Crypt::SSLeay;  
-            Crypt::SSLeay->import();
-         };
-         if ($@) {
-            die "Crypt::SSLeay is required for https connections, but could not be loaded: $@";
-         }
-      }
+   if ($url =~ s|http(s?)://(.*)/sdk.*|http$1://$2/sdk/vimService.wsdl|i) {
+      # if ($1 eq "s") {
+         # eval {
+            # require Crypt::SSLeay;
+            # Crypt::SSLeay->import();
+         # };
+         # if ($@) {
+            # die "Crypt::SSLeay is required for https connections, but could not be loaded: $@";
+         # }
+      # }
 
       my $temp_addr = $2;
       if ($temp_addr =~ /:/) {
          if (($temp_addr =~ tr/:/:/) > 1) {
-            die "Unsupported IP address format\n";
+            require Net::INET6Glue::INET_is_INET6;
          }
       }
 
@@ -502,47 +643,59 @@ sub query_api_supported {
 sub query_server_version {
    BEGIN {
       #To remove SSL Warning, switching from IO::Socket::SSL to Net::SSL
-      $ENV{PERL_NET_HTTPS_SSL_SOCKET_CLASS} = "Net::SSL";
+      # $ENV{PERL_NET_HTTPS_SSL_SOCKET_CLASS} = "Net::SSL";
 	  #To remove host verification
       $ENV{PERL_LWP_SSL_VERIFY_HOSTNAME} = 0;
    }
    my $url = shift;
    if ($url =~ s|http(s?)://(.*)/sdk.*|http$1://$2/sdk/vimService.wsdl|i) {
       # bug 288336
-      if ($1 eq "s") {
-         eval {  
-            require Crypt::SSLeay;  
-            Crypt::SSLeay->import();
-         };
-         if ($@) { 
-            die "Crypt::SSLeay is required for https connections, but could not be loaded: $@";
-         }
-      }
-      # no IPv6 support yet
+      # if ($1 eq "s") {
+         # eval {
+            # require Crypt::SSLeay;
+            # Crypt::SSLeay->import();
+         # };
+         # if ($@) {
+            # die "Crypt::SSLeay is required for https connections, but could not be loaded: $@";
+         # }
+      # }
+
       my $temp_addr = $2;
       if ($temp_addr =~ /:/) {
          if (($temp_addr =~ tr/:/:/) > 1) {
-            die "Unsupported IP address format\n";
+            require Net::INET6Glue::INET_is_INET6;
          }
       }
+
       my $user_agent = LWP::UserAgent->new(agent => "VI Perl");
       my $cookie_jar = HTTP::Cookies->new(ignore_discard => 1);
       $user_agent->cookie_jar($cookie_jar);
       $user_agent->protocols_allowed(['http', 'https']);
-  
+
       my $http_header = HTTP::Headers->new(Content_Type => 'text/xml');
       my $request = HTTP::Request->new('GET', $url);
-     
-      my $response = $user_agent->request($request);   
-     
+
+      my $response = $user_agent->request($request);
+
       if ($response->content =~ /Connection refused/) {
          die $response->content . "\n";
       } elsif ($response->content =~ /Bad hostname/) {
-         die $response->content . "\n";
+         # Loading IPv6 module, to check hostname
+         eval {
+              require Net::INET6Glue::INET_is_INET6;
+         };
+         if ($@){
+            die $response->content . "\n";
+         }
+         $response = $user_agent->request($request);
+         # Still not able to resolve hostname
+         if ($response->content =~ /Bad hostname/) {
+           die $response->content . "\n";
+         }
       } elsif ($response->content =~ /Connect failed/) {
          die $response->content . "\n";
       }
-      
+
       my $xml_parser = XML::LibXML->new;   
       my $result;   
 
@@ -872,7 +1025,7 @@ sub validate_vim_soap {
    my $soap_env_body = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
    $soap_env_body = $soap_env_body."<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">";
    $soap_env_body = $soap_env_body."<soapenv:Body>";
-   $soap_env_body = $soap_env_body."<CurrentTime xmlns=\"urn:vim2\">";
+   $soap_env_body = $soap_env_body."<CurrentTime xmlns=\"urn:vim25\">";
    $soap_env_body = $soap_env_body."<_this type=\"ServiceInstance\">ServiceInstance</_this>";
    $soap_env_body = $soap_env_body."</CurrentTime>";
    $soap_env_body = $soap_env_body."</soapenv:Body>";
@@ -1001,6 +1154,10 @@ sub load_session {
       chomp $line;
       $server = $1 if $line =~ /\w*\s*domain=(.*).local/;
       $server = $1 if $line =~ /\w*\s*domain=(.*?);/;
+      #check for IPv6
+      if (($server =~ tr/:/:/) > 1) {
+         $server = $1 if $line =~ /\w*\s*domain="(.*).local/;
+      }
       close SFILE or die $!;
    }
    $url = Opts::construct_service_url(
@@ -1609,6 +1766,8 @@ sub logout {
 ##################################################################################
 package ViewBase;
 
+use Encode;
+
 sub new {
    my ($class, $mo_ref, $vim) = @_;
    if (! $mo_ref) {
@@ -1658,6 +1817,10 @@ sub update_view_data {
    my $self = shift;
    my $properties = shift;
    my $service = $self->{vim}->get_vim_service();
+   if ($^O =~ /MSWin32/) {
+      #to remove SOAP error caused by utf8 character
+      $self->{mo_ref}->{value} = encode_utf8($self->{mo_ref}->{value});
+   }
    my $property_filter_spec = (ref $self)->get_property_filter_spec($self->{mo_ref});
    my $propertyCollector = $self->{vim}->get_service_content()->propertyCollector;
    if (defined($properties) && $properties ne "") {
@@ -1711,6 +1874,9 @@ sub get_property {
          $path = $1;
          unshift @subpaths, $2;
       }
+      else { 
+         last;
+     }
    }
    if (defined($val)) {
       foreach (@subpaths) {
@@ -2143,8 +2309,8 @@ sub unload_session {
 sub get_session_loaded {
    my $self = shift;
    my $user_agent = $self->{user_agent};
-   return defined $user_agent->cookie_jar and 
-                  $user_agent->cookie_jar->as_string ne '';
+   return (defined $user_agent->cookie_jar and
+                  $user_agent->cookie_jar->as_string ne '');
 }
 
 
@@ -2159,9 +2325,14 @@ sub get_session_id {
 sub load_session_id {
    my ($self, $id) = @_;
    my $user_agent = $self->{user_agent};
-   # bug 264430
    my $url = URI->new($self->{url});
    my $host = $url->host();
+   # If host name is in IPv6 format, setting domain name as "IPv6 address.local"
+   if (($host =~ tr/:/:/) > 1){
+      if (!($host =~ m/.local/)){
+         $host = $host . ".local";
+      }
+   }
    $id = '"' . $id . '"';
    my %hash = ();
    $user_agent->cookie_jar->set_cookie(
@@ -2177,12 +2348,14 @@ sub request {
    if (!$soap_action) {
       $soap_action = '""';
    }
+   $soap_action =~ s/\s//g;
 
    my $namespace = Vim::get_vim_namespace;
    confess "Not logged in and/or no XML namespace set" unless $namespace;
    my $request_envelope =
       "$soap_header<$op_name xmlns=\"urn:$namespace\">$body_content</$op_name>$soap_footer";
-
+   #downgrading the request envelope to work with utf8 character
+   utf8::downgrade($request_envelope);
    # http header
    my $http_header = HTTP::Headers->new(
                         Content_Type => 'text/xml',
@@ -2308,6 +2481,27 @@ sub val {
 ##################################################################################
 package ComplexType;
 
+#hash for anytype class
+our %any_type = (
+    "ClusterRuleSpec" => "xsd:int",
+);
+
+#to get the object/type defined for anyType class
+sub get_anytype {
+   my $val = shift;
+   if( exists($any_type{$val}) ) {
+      return $any_type{$val};
+   } else {
+      return undef;
+   }
+}
+
+#to set the object and type defined for anyType class
+sub update_anytype {
+   my ($object,$type)=@_;
+   $any_type{$object} = $type;
+}
+
 sub deserialize {
    my ($class, $element_node) = @_;
    if (! defined($element_node)) {
@@ -2322,16 +2516,16 @@ sub deserialize {
    my $self = {};
    my @property_list = $class->get_property_list();
    foreach (@property_list) {
-      my ($property_name, $class_name, $isarray) = @$_; 
+      my ($property_name, $class_name, $isarray) = @$_;
       my @child_nodes = $element_node->getChildrenByTagName($property_name);
       if (! @child_nodes) {
          next;
-      }      
-      my $property_val = [];   
+      }
+      my $property_val = [];
       foreach (@child_nodes) {
          my $val;
          my $child_class_name = $class_name;
-         
+
          if (defined $child_class_name) {
             my $child_type_node =
                $_->getAttributeNodeNS('http://www.w3.org/2001/XMLSchema-instance', 'type');
@@ -2385,10 +2579,10 @@ sub serialize {
    if ($emit_type) {
       my $type_name = ref $self;
       $serialized_string = "<$tag xsi:type=\"$type_name\">";
-   } 
+   }
    foreach (@property_list) {
       # bug 193402
-      my ($property_name, $class_name, $isarray, $mandatory) = @$_;      
+      my ($property_name, $class_name, $isarray, $mandatory) = @$_;
       if (! exists($self->{$property_name})) {
          if ($mandatory == 1) {
             Carp::confess "Mandatory property \"$property_name\" missing";
@@ -2413,27 +2607,34 @@ sub serialize {
                } else {
                   Carp::confess "Cannot serialize boolean: acceptable values are true/false/1/0, got '$_'";
                }
-               $serialized_string .= "<$property_name>" . XmlUtil::escape_xml_string($val) . "</$property_name>";                          
+               $serialized_string .= "<$property_name>" . XmlUtil::escape_xml_string($val) . "</$property_name>";
                next;
             }
-            
+
             # complex type
             my $show_type;
             my $obj_type_name = ref $_;
-            if ($class_name ne $obj_type_name) {               
+            if ($class_name ne $obj_type_name) {
                $show_type = 1;
                if ($class_name eq 'anyType' &&  ! $obj_type_name) {
-                  # primitive going out as any -- treat them as xsd:string
                   if (defined ($_)) {
-                     $serialized_string .=       
+                  my @any_type_obj = split(/\=/, $self);
+                  if(get_anytype($any_type_obj[0])){
+                      $serialized_string .=
+                               "<$property_name xsi:type=\"".get_anytype($any_type_obj[0])."\">" .
+                               XmlUtil::escape_xml_string($_) . "</$property_name>";
+                  }
+                  else{
+                     $serialized_string .=
                         "<$property_name xsi:type=\"xsd:string\">" .
                         XmlUtil::escape_xml_string($_) . "</$property_name>";
+                     }
                   }
                   next;
                }
             }
             if ($obj_type_name && $class_name eq 'ManagedObjectReference') {
-               $obj_type_name = 'ManagedObjectReference';   
+               $obj_type_name = 'ManagedObjectReference';
             }
             if ($class_name ne 'anyType') {
                if (! $obj_type_name || ! $obj_type_name->isa($class_name)) {
@@ -2447,16 +2648,15 @@ sub serialize {
          } else {
             # primitive
             if (defined($_)) {
-               $serialized_string .=       
-                  "<$property_name>" . XmlUtil::escape_xml_string($_) . "</$property_name>";     
-            }         
+               $serialized_string .=
+                  "<$property_name>" . XmlUtil::escape_xml_string($_) . "</$property_name>";
+            }
          }
       }
    }
    return $serialized_string .= "</$tag>";
 }
 
-
 sub new {
    my ($class, %args) = @_;
 
