@@ -1,5 +1,5 @@
 #
-# Copyright 2006 VMware, Inc.  All rights reserved.
+# Copyright 2006,2016,2017,2019 VMware, Inc.  All rights reserved.
 #
 
 use 5.006001;
@@ -12,8 +12,8 @@ use File::Basename;
 package Util;
 our $tracelevel = 0;
 our $tracefilter;
-our $version = '5.5.0';
-our $script_version = '5.5.0';
+our $version = '7.0.0';
+our $script_version = '7.0.0';
 
 
 sub trace {
@@ -134,6 +134,11 @@ our %options = (
       help => "Encoding: utf8, cp936 (Simplified Chinese), iso-8859-1 (German), shiftjis (Japanese)",
       default => "utf8",
    },
+   psc => {
+      type => "=s",
+      variable => "VI_PSC",
+      help => "Platform services controller's hostname or IP",
+   },
 );
 
 our @builtin_options = keys %options;
@@ -157,6 +162,43 @@ sub get_option {
    my ($key) = @_;
    Util::trace(5, "get_option $key\n");
    my $opt = $options{$key};
+   #
+   # Block to remove Perl Bug related to Net-HTTP module: IPv6 Address URLs Broken.
+   # For details URL: https://rt.cpan.org/Public/Bug/Display.html?id=75618
+   #
+   if (defined($opt->{variable})){
+      if (defined($opt->{value}) && $opt->{variable} eq "VI_URL" ){
+         my $Url = $opt->{value};
+         $Url =~ s|http(s?)://(.*)/sdk.*|http$1://$2/sdk(.*)|i;
+         my $protocol = $1;
+         my $server_from_url = $2;
+         if (($server_from_url =~ tr/:/:/) > 1) {
+            if ($server_from_url =~ /]]/){
+               #do nothing
+            }
+            elsif ($server_from_url =~ /]/){
+               if ($^O =~ /MSWin32/) {
+                  $server_from_url = "[" . $server_from_url . "]";
+               }
+            }
+            else{
+               if ($^O =~ /MSWin32/) {
+                  $server_from_url = "[[" . $server_from_url . "]]";
+               }
+               else {
+                  $server_from_url = "[" . $server_from_url . "]";
+               }
+            }
+         }
+         if (defined($protocol)) {
+            $opt->{value} = "https://" . $server_from_url . "/sdk";
+         }
+         else {
+            $opt->{value} = "http://" . $server_from_url . "/sdk";
+         }
+      }
+   }
+
    if (!defined($opt)) {
       Util::trace(5, "  => unknown option\n");
       return undef;
@@ -241,6 +283,25 @@ sub construct_service_url {
    $server ||= get_option('server');
    $port ||= get_option('portnumber');
    $path ||= get_option('servicepath');
+   # For IPv6
+   if (($server =~ tr/:/:/) > 1) {
+      if ($server =~ /]]/){
+         #do nothing
+      }
+      elsif ($server =~ /]/){
+         if ($^O =~ /MSWin32/) {
+            $server = "[" . $server . "]";
+         }
+      }
+      else{
+         if ($^O =~ /MSWin32/) {
+            $server = "[[" . $server . "]]";
+         }
+         else {
+            $server = "[" . $server . "]";
+         }
+      }
+   }
    my $url_out = URI->new;
    $url_out->scheme($protocol);
    $url_out->host($server);
@@ -253,9 +314,36 @@ sub construct_service_url {
 sub get_url {
    my $protocol = get_option('protocol');
    my $server = get_option('server');
+   my $lsurl = undef;
+   my $pscurl = Opts::get_option('psc');
+   if (defined($pscurl)){
+      $lsurl = "https://". $pscurl . "/lookupservice/sdk";
+      my $lookupservice = new LookupService(lookup_url => $lsurl);
+      my %results = $lookupservice->get_default_mgmt_node();
+      my @nodes_ids = values %results;
+      my $node_id = $nodes_ids[0];
+      my $url = $lookupservice->find_vim_url($node_id);
+      return $url;
+   }
    my $port = get_option('portnumber');
    $port = defined($port) ? ":$port" : "";
    my $path = get_option('servicepath');
+   #For IPv6
+   if (($server =~ tr/:/:/) > 1) {
+       if ($server =~ /]/){
+          if ($^O =~ /MSWin32/) {
+             $server = "[" . $server . "]";
+          }
+       }
+       else{
+          if ($^O =~ /MSWin32/) {
+             $server = "[[" . $server . "]]";
+          }
+          else {
+             $server = "[" . $server . "]";
+          }
+       }
+    }
    return "${protocol}://${server}${port}${path}";
 }
 
@@ -610,7 +698,8 @@ sub validate {
          }
       }
 
-      if ((!option_is_set('username') || !option_is_set('password') || 1 == $askPwd) && !option_is_set('sessionfile')) {
+      if ((!option_is_set('username') || !option_is_set('password') || 1 == $askPwd)
+          && (!option_is_set('sessionfile') )) {
          # First Check It In The Credential Store Library
          # bug 374359
          my $servername;
@@ -709,9 +798,15 @@ sub validate {
          }
       }
    }
-   
+
    # one of sessionfile or (username, password) must be present. if not print usage
-   if ($valid && !option_is_set('sessionfile') && 
+   if (!option_is_set('ssourl') &&
+       option_is_set('ssoauth')) {
+      print "The 'ssourl' option must set when 'ssoauth' is enabled";
+   }
+
+   # one of sessionfile or (username, password) must be present. if not print usage
+   if ($valid && !option_is_set('sessionfile') &&
        !(option_is_set('username') && option_is_set('password')) &&
        !(option_is_set('passthroughauth'))) {
       print "Must have one of command options 'sessionfile', 'passthroughauth', 'credstore' or " .
